%{
==========================================================================
---------  CODICE PER LA SOLUZIONE APPROSSIMATA ARCO-IMPULSO  -------------
----------------------  CON PERTURBAZIONE J2  -----------------------------
Soluzione per tempo minimo, modificato per tner conto della diminuzione
di massa
===========================================================================
Federico Feduzi s305851
Ultimo Aggiornamento = 07/01/2025

------- NOTE ---------
I Delta a, Delta i, Delta Omega e Delta t vengono iseriti dall'utente da
tastiera; per cambiare l'orbita di partenza andare nella sezione " ORBITA
PERTENZA"
===========================================================================
%}

clear all; 

%% ======  COSTANTI  ======
mu=398600.4418;                                                                             % Costante gravitazionale terrestre [km^3/s^2]
J2=1.08262668*1e-3;                                                                         % Costante J2
R_e=6378.1363;                                                                              % Raggio Terrestre [km]

V_ref= sqrt(mu/R_e);                                                                        % Velocitá di riferimento [km/s]
t_ref= R_e/V_ref;                                                                           % Tempo di riferimento [s]

%% ======  PARAMETRI DI INPUT  ======
disp("===============================================================================")
disp("Programma che calcola il Delta V ottimo per il trasferimento tra due orbite LEO")
disp("tramite approssimazione Arco-impulso, con effetto di J2, Modificato")
disp("-------------------------------------------------------------------------------")
disp("Soluzione a tempo minimo")
disp("-------------------------------------------------------------------------------")
disp("Parametri di input:")
Delta_a = input("Delta a [km] = ");                                                         % Delta a [km]
Delta_i = deg2rad(input("Delta i [deg] = "));                                               % Delta i [rad] (input in [deg])
Delta_Omega = deg2rad(input("Delta Omega [deg] = "));                                       % Delta Omega [rad] (input in [deg])
Delta_t = 700;                                                                              % Durata trasferta [adimensionale] (primo tentativo, Mettere a 0 per manovre brevi)
eps_t = input("Incremento Delta t adim per ogni iterazioni = ");                            % Incremento adim del Delta_t per ogni iter per torvare tmin  
disp("-------------------------------------------------------------------------------")

%% ======  ORBITA PARTENZA  =======
h = 400;                                                                                    % Altezza Orbita [km]
a_0 = R_e+h;                                                                                % Semiasse maggiore [km]
i_0 = deg2rad(51);                                                                          % Inclinazione [rad]
Omega_0 = deg2rad(0);                                                                       % Omega [rad]
e_0 = 0;                                                                                    % Eccentricitá

%% ======  ORBITA TARGET  =======
a_f = a_0+Delta_a;                                                                          % Semiasse maggiore [km]
i_f = i_0+Delta_i;                                                                          % Inclinazione [rad]
Omega_f = Omega_0+Delta_Omega;                                                              % Omega [rad]
e_f = 0;                                                                                    % Eccentricitá

%% ======  PARAMETRI MOTORE  =======
T = 0.01;                                                                                   % Spinta motore [N]
m0 = 15;                                                                                    % Massa Spacecraft [kg]
a0= T/m0/1000;                                                                              % Accelerazione [km/s^2]
c = 2500.d0*9.80665/1000;                                                                   % Impulso Spec [km/s]

%% ====== SOLUZIONE ==========
DV = [0,0];                                                                                 % Delta V 2 impulsi [Delta V_a, Delta V_b]  [km/s]
DV1 = [0,0];                                                                                % Delta V solo impulso all'arrivo [0, Delta V_b]  [km/s]
DV2 = [0,0];                                                                                % Delta V solo impulso alla partenza [Delta V_a, 0]  [km/s]

eps = 1e-8;                                                                                 % Tolleranza relativa
Flag2=0;                                                                                    % Falg per trovare il tempo minimo

while Flag2==0
    err =1;                                                                                 % Inizializzazione errore
    j =0;                                                                                   % Inizializzazione n di iteazioni
    t_a = 0;                                                                                % Inizializzazione t_a = DV_a/a
    t_b = 0;                                                                                % Inizializzazione t_b = DV_b/a
    DV_salvato=[0,0];                                                                       % Delta V salvato dal passo precedente
    Flag = 0;                                                                               % inizializzazione Flag
    
    % Ciclo di iterazione principale
    while err>eps && Flag==0
       
        Delta_t_dim = Delta_t*t_ref;                                                        % Durata trasferta [s]
        t_tras = Delta_t_dim -t_a/2 -t_b/2;                                                 % Calcolo durata trasferta (tra i due impulsi)
        
        % Funzione che valuta i Delta V per le 3 manovre impulsive
        [DV(1),DV(2),DV1(1),DV1(2),DV2(1),DV2(2)] = det_deltaV(a_0,a_f,i_0,i_f,0,0,0,0,Omega_0,Omega_f,t_a/2+t_tras,t_a/2);
        [DV_min,I] = min([sum(DV),sum(DV1),sum(DV2)]);                                      % Determinazione Delta V minore 
    
        % Selezione della manovra minore
        if I==1
            DV_sel = DV;
        elseif I==2
            DV_sel = DV1;
        else 
            DV_sel = DV2;
        end
        %DV_sel =DV;
        % Controllo della fattibilitá della manovra
        % if sum(DV_sel)/a0/t_ref > Delta_t
        %    Flag=1
        % end
        
        err = norm(DV_sel - DV_salvato)/norm(DV_sel);                                       % Calcolo errore rispetto iterazione porecedente

        % t_a = DV_sel(1)/a0;                                                                  % Calcolo di t_a
        % t_b = DV_sel(2)/a0;                                                                  % Calcolo di t_a
        
        m_a = m0*exp(-DV_sel(1)/c);                                                             % massa dopo il primo impuslo
        a_1 = T/(0.5*(m0+m_a))/1000;                                                            % Accelerazione media primo imp.
        m_b = m_a*exp(-DV_sel(2)/c);                                                            % massa dopo il secondo impulso
        a_2 = T/(0.5*(m_b+m_a))/1000;                                                           % Accelerazione media secondo imp.
        t_a =  DV_sel(1)/a_1;
        t_b = DV_sel(2)/a_2;

        DV_salvato = DV_sel;                                                                % Memorizzazione
        j=j+1;

           % Controllo della fattibilitá della manovra
        if (t_a/t_ref + t_b/t_ref) > Delta_t
           Flag=1;
        end
    end

    if Flag == 0                                                                            % Se Flag == 0 ho trovato il tempo min
        Flag2=1;
    else                                                                                    % Antrimenti rifaccio incrementando il delta_t
        Delta_t = Delta_t+eps_t;
    end
    
end

%% ====== OUTPUT DEL CODICE ============
DV_ottenuto = sum(DV_salvato);                                                              % Delta V ottenuto [km/s]
t_adim = t_tras/t_ref;                                                                      % Tempo tra i 2 impulsi [adim]                            
%t_spint = DV_ottenuto/a0/t_ref;                                                            % Tempo di spinta [adim]
t_spint = DV_salvato(1)/a_1/t_ref + DV_salvato(2)/a_2/t_ref;
rapporto = t_spint/Delta_t;                                                                 % Rapporto tra il tempo di spinta e la durata della manovra
%mf = exp(-sum(DV_salvato)/c);
mf = m_b/m0;
manovra =["2 impulsi ", "impulso solo all'arrivo","impulso solo alla partenza"];

disp("Il tempo minimo adimensionale è = "+Delta_t)
disp("Delta V = "+DV_ottenuto)
disp("Ottenuto con manovra = "+ manovra(I))
disp("Tempo di Spinta adimensionale = "+ t_spint)
disp("Rapporto tra t spinta e Delta t = "+rapporto)
disp("===============================================================================")