function[delta_va,delta_vb,delta_va1,delta_vb1,delta_va2,delta_vb2]=det_deltaV(a1,a2,i1,i2,e1,e2,ep1_f,ep2_f,Omega_ef1,Omega_ef2,ta,tp)
%% ========================================================================
% Funzione per il clacolo del Delta V impulsivo per una manovra a 2
% impulsi con metodo approssimato J2-perturbato.
% 
% MODIFICATO : 10/01/2025
%
% INPUT:
% a1 = semiasse orbita 1            [km]
% a2 = semisasse orbita 2           [km]
% i1 = inclinazione orbita 1        [rad]
% i2 = inclinazione orbita 2        [rad]
% e1 = eccentricità orbita 1
% e2 = eccentricità orbita 2
% ep1_f = epoca param. orbita 1     [s]
% ep2_f = epoca param. orbita 2     [s]
% Omega_ef1 = Omega eff. orbita 1   [rad]
% Omega_ef2 = Omega_eff. orbita 2   [rad]
% ta = tempo di arrivo              [s]
% tp = tempo di partenza            [s]
% 
% OUTPUT:
% Solutione 2 impulsi:
% delta_va = impulso alla partenza              [km/s]
% delta_vb = impulso all'arrivo                 [km/s]
% Solutione 1 impulso nel punto finale :
% delta_va1 = impulso alla partenza (0)         [km/s]
% delta_vb1= impulso all'arrivo                 [km/s]
% Solutione 1 impulso nel punto iniziale :
% delta_va2 = impulso alla partenza             [km/s]
% delta_vb2= impulso all'arrivo (0)             [km/s]

%% =======================================================================


%% ==== Costanti ====
mi=398600.4415;
J2=1.08262668*1e-3;
re=6378.1363;


%% === Calcolo del rateo di omega ===
Omega_dot1=-(3/2)*J2*(re/(a1*(1-e1^2)))^2*sqrt(mi/a1^3)*cos(i1);             % [rad/s]
Omega_dot2=-(3/2)*J2*(re/(a2*(1-e2^2)))^2*sqrt(mi/a2^3)*cos(i2);                 % [rad/s]

%% === Calcolo del RAANS alla partenza ===
Omega01=wrapTo2Pi(Omega_dot1*(tp-ep1_f)+Omega_ef1);                         % Orbita 1
Omega02=wrapTo2Pi(Omega_dot2*(tp-ep2_f)+Omega_ef2);                         % Orbita 2

Delta_omega=wrapToPi(Omega02-Omega01);                                      % Calcolo Delta Omega
Delta_omegadot=Omega_dot2-Omega_dot1;                                       % Calcolo Delta Omega dot

deltat=-(Delta_omega)/(Delta_omegadot);                                     % Calcolo del t*
if deltat<0
    deltat=deltat+2*pi/abs(Delta_omegadot);
end

t=tp+deltat;

%% === Quantitá medie ===
i0=(i2+i1)/2;                                                               % Calcolo inclinazione media
a0=(a2+a1)/2;                                                               % Calcolo semiasse medio
v0=sqrt(mi/a0);                                                             % Velocità media orbitale

%% === Calcolo del RAANS all'arrivo
Omega1=wrapToPi(Omega01+Omega_dot1*(ta-tp));                                
Omega2=wrapToPi(Omega02+Omega_dot2*(ta-tp));

Delta_omega=wrapToPi(Omega02-Omega01);                                      % Delta Omega Partenza
Delta_omega_ar=wrapToPi(Omega2-Omega1);                                     % Delta Omega arrivo

%% ===  Calcolo Componendi DV ===
x0=(pi/2)*(Delta_omega)*sin(i0)*v0;                                         % x alla parteza
x=(pi/2)*(Delta_omega_ar)*sin(i0)*v0;                                       % x all'arrivo
y=(a2-a1)*v0/(2*a0);
z=(pi/2)*(i2-i1)*v0;

Omega_dot0=(Omega_dot1+Omega_dot2)/2;

m=(pi/2)*7*Omega_dot0*sin(i0)*(ta-tp);

n=Omega_dot0*tan(i0)*sin(i0)*(ta-tp);

m1=7*Omega_dot1*sin(i1)*(ta-tp);

n1=Omega_dot1*tan(i1)*sin(i1)*(ta-tp);

s_x_x=(2*x+m*y+n*z)/(4+m^2+n^2);

s_y_y=-(2*m*x-(4+n^2)*y+m*n*z)/(8+2*m^2+2*n^2);

s_z_z=-(2*n*x+m*n*y-(4+m^2)*z)/(8+2*m^2+2*n^2); %%

delta_x=m*s_y_y+n*s_z_z;

delta_va=sqrt((s_x_x)^2+(s_y_y)^2+(s_z_z)^2);

delta_vb=sqrt((x-s_x_x + delta_x)^2+(y-s_y_y)^2+(z-s_z_z)^2);   

t;
ta;

% if t<ta
%     delta_va=0.5*sqrt(y^2+z^2);
%     delta_vb=delta_va;
% end  

%Ciclo IF modificato
if t<ta
    s_x = 0;
    s_y = 1-t/ta;
    s_z = 1-t/ta;
    delta_va=sqrt((s_y*y)^2+(s_z*z)^2);
    delta_vb=sqrt((y-s_y*y)^2+(z-s_z*z)^2);

end


%Impulso nel punto finale, sol attendista
delta_va1=0;
delta_vb1=sqrt(x^2+y^2+z^2);



%Impulso nel punto iniziale
delta_vb2=0;
delta_va2=sqrt((x0)^2+y^2+z^2);

%[s_x,s_y,s_z]
%[x,y,z,s_x_x,s_y_y,s_z_z,delta_x, delta_va,delta_vb]
end
